import numpy as np
import pandas as pd
from pykrige.ok import OrdinaryKriging
import os
import matplotlib.pyplot as plt
import sys

def dbm_to_watt(pdbm):
    return (10 ** (pdbm / 10)) / 1000

def ABOI(measurements, interpolation_grid):
    OK = OrdinaryKriging(measurements.x,measurements.y, measurements.watt, variogram_model="exponential",
                         verbose=False, enable_plotting=False)
    interpolation, ss = OK.execute('points', interpolation_grid.x.astype(np.float64), interpolation_grid.y.astype(np.float64))
    return np.mean(interpolation)


def boxplot_experiments(results,filename, expected_value, sample_sizes):
    plt.figure(figsize=(3.5, 3.5))
    plt.boxplot(results, whis=2.5,showfliers=False)
    plt.axhline(y=expected_value, ls='--', color='r')
    plt.ylim(0, 30 * 1e-12)
    plt.xticks(range(1, len(sample_sizes) + 1), sample_sizes)
    plt.yticks([0,5,10,15,20,25,30]*np.repeat(1e-12,7), [0,5,10,15,20,25,30])
    plt.grid()
    plt.xlabel('Number of Measurements')
    plt.ylabel('Signal Strength [pW]')
    plt.tight_layout()
    plt.savefig(filename)
    return True

def main(simulated, uniform):
    if simulated:
        gt_file = 'simulated_ground_truth.csv'
        sample_sizes = [50, 100, 200, 400, 700, 1000]
        if uniform:
            pos_dir = 'simulated_uniform'
            experiment = 'simulated_uniform'
        else:
            pos_dir = 'simulated_nonuniform'
            experiment = 'simulated_nonuniform'
    else:
        gt_file = 'real_ground_truth.csv'
        sample_sizes = [25, 50, 100, 200, 350, 500]
        if uniform:
            pos_dir = 'real_uniform'
            experiment = 'real_uniform'
        else:
            pos_dir = 'real_nonuniform'
            experiment = 'real_nonuniform'

    gt_data = pd.read_csv(gt_file, sep=',', header=0)
    gt_data['watt'] = gt_data.dbm.apply(dbm_to_watt)
    gt_grid = np.full((np.max(gt_data.x) + 1, np.max(gt_data.y) + 1), np.nan)
    for index, row in gt_data.iterrows():
        gt_grid[int(row.x)][int(row.y)] = row.watt

    ABOI_box = []
    mean_box = []
    median_box = []

    expected_value = np.mean(gt_data.watt)
    
    print("mean_rmse", "median_rmse", "ABOI_rmse")    # rmse headers
    
    for n in sample_sizes:
        ABOI_results = []
        mean_results = []
        median_results = []
        pos_files = [f for f in os.listdir(pos_dir) if 'positions_{}_'.format(n) in f]
        for file in pos_files:
            try:

                measurements = pd.read_csv(pos_dir+'/'+file, sep=',', header=0)
                measurements['watt'] = ''
                watts = []
                for index, row in measurements.iterrows():
                    watts.append(gt_grid[int(row.x)][int(row.y)])
                    
                measurements['watt'] = watts
                ABOI_results.append(ABOI(measurements, gt_data))
                mean_results.append(np.mean(measurements.watt))
                median_results.append(np.median(measurements.watt))
            except:
                print(file)

        ABOI_box.append(ABOI_results)
        mean_box.append(mean_results)
        median_box.append(median_results)
        
        mean_rmse = (sum(map(lambda x: (x-expected_value)**2, mean_results)) / 20)**(1/2)
        median_rmse = (sum(map(lambda x: (x-expected_value)**2, median_results)) / 20)**(1/2)
        ABOI_rmse = (sum(map(lambda x: (x-expected_value)**2, ABOI_results)) / 20)**(1/2)
        print(mean_rmse, median_rmse, ABOI_rmse)
        
    boxplot_experiments(ABOI_box, 'ABOI_{}.pdf'.format(experiment), expected_value, sample_sizes)
    boxplot_experiments(mean_box, 'arithmetic_mean_{}.pdf'.format(experiment),expected_value, sample_sizes)
    boxplot_experiments(median_box, 'median_value_{}.pdf'.format(experiment), expected_value, sample_sizes)

if __name__ == '__main__':
    args = sys.argv[1:]
    assert len(args) == 2

    assert args[0] in ["simulated", "real"]
    simulated = (args[0] == "simulated")

    assert args[1] in ["uniform", "nonuniform"]
    uniform = (args[1] == "uniform")

    main(simulated, uniform)
